﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using com.citizen.sdk;
using com.citizen.sdk.poslayout;

//-------------------------------------------------------------------------------
// (C) 2009-2021 CITIZEN SYSTEMS JAPAN CO., LTD. All Rights Reserved.
//-------------------------------------------------------------------------------
// Layout SDK (via POS Print SDK), sample code [[ ReceiptPrint ]] for VCS2010
//
// This program assumed receipt print.
// It can easily change a print result by replacing a layout file.
// The following layout files are necessary for the execution of this program.
//
// CLF Layout File - PC font printing (Full graphic printing).
//   sample_48mm.CLF
//   sample_72mm.CLF
//   sample_48mm_landscape.CLF
//   sample_72mm_landscape.CLF
// XML Layout File - Printer font printing.
//   sample_48mm.XML
//   sample_72mm.XML
//   sample_48mm_landscape.XML
//   sample_72mm_landscape.XML
//
// In addition, please understand that error processing is not supported.
namespace ReceiptPrint
{
	public partial class Form1 : Form
	{
		private com.citizen.sdk.poslayout.PrintOption printOption = new com.citizen.sdk.poslayout.PrintOption();
		ConnectTypeList connectTypeList = new ConnectTypeList();

		string replaceLogo = @"..\..\newlogo.png";

		public Form1()
		{
			InitializeComponent();
		}

		private void Form1_Load(object sender, EventArgs e)
		{
			// Set the Layout file directory.
			String dir = System.IO.Path.Combine(Application.StartupPath, @"..\..\");
			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(dir);

			// Set the Layout file list.
			comboBox_LayoutFile_CLF.Items.AddRange(dirInfo.GetFiles("*.CLF"));
			if (comboBox_LayoutFile_CLF.Items.Count > 0)
				comboBox_LayoutFile_CLF.SelectedIndex = 0;
			comboBox_LayoutFile_XML.Items.AddRange(dirInfo.GetFiles("*.XML"));
			if (comboBox_LayoutFile_XML.Items.Count > 0)
				comboBox_LayoutFile_XML.SelectedIndex = 0;

			// Set the connection type list.
			comboBox_ConnectType.DataSource = connectTypeList;
			comboBox_ConnectType.DisplayMember = "Value";
			comboBox_ConnectType.ValueMember = "Key";
			comboBox_ConnectType.SelectedValue = ESCPOSConst.CMP_PORT_USB;

			// Set EventHandler
			button_Print.Click += new System.EventHandler(button_Print_Click);
			button_SearchPrinter.Click += new System.EventHandler(button_SearchPrinter_Click);
			button_PrintOption.Click += new System.EventHandler(button_PrintOption_Click);
			checkBox_PrintOption.CheckedChanged += new System.EventHandler(checkBox_PrintOption_CheckedChanged);
			radioButton_Layout_CLF.CheckedChanged += new System.EventHandler(radioButton_Layout_CLF_CheckedChanged);
			comboBox_ConnectType.SelectedIndexChanged += new System.EventHandler(comboBox_ConnectType_SelectedIndexChanged);

			// Set property.
			checkBox_PrintOption.Checked = true;
			radioButton_Layout_CLF.Checked = true;

			// List the printers (ports).
			listPrinterPort();
		}


		private string getSelectedLayoutFile()
		{
			if (radioButton_Layout_CLF.Checked)
			{
				int index = comboBox_LayoutFile_CLF.SelectedIndex;
				if (index < 0) return null;
				System.IO.FileInfo fileInfo = (System.IO.FileInfo)comboBox_LayoutFile_CLF.Items[index];
				return fileInfo.FullName;
			}
			else
			{
				int index = comboBox_LayoutFile_XML.SelectedIndex;
				if (index < 0) return null;
				System.IO.FileInfo fileInfo = (System.IO.FileInfo)comboBox_LayoutFile_XML.Items[index];
				return fileInfo.FullName;
			}
		}

		private string getSelectedPrinter()
		{
			if (listView_Printer.SelectedItems.Count == 0)
			{
				return "";
			}
			return listView_Printer.SelectedItems[0].Text;
		}

		private PrintOption getPrintOption()
		{
			PrintOption option = null;
			if (checkBox_PrintOption.Checked)
			{
				option = this.printOption;
			}
			return option;
		}

		private int getSelectedConnectType()
		{
			// for using POS Print SDK : 
			return (int)comboBox_ConnectType.SelectedValue;
		}



		private void radioButton_Layout_CLF_CheckedChanged(object sender, EventArgs e)
		{
			bool isChecked = ((RadioButton)sender).Checked;
			comboBox_LayoutFile_CLF.Enabled = isChecked;
			comboBox_LayoutFile_XML.Enabled = !isChecked;
		}

		private void checkBox_PrintOption_CheckedChanged(object sender, EventArgs e)
		{
			bool isChecked = ((CheckBox)sender).Checked;
			button_PrintOption.Enabled = isChecked;
		}

		private void button_PrintOption_Click(object sender, EventArgs e)
		{
			this.Enabled = false;
			Form_PrintOption f = new Form_PrintOption();
			f.setPrintOption(this.printOption);
			if (f.ShowDialog(this) == DialogResult.OK)
			{
				PrintOption option = f.getPrintOption();
				this.printOption = option.Clone();
			}
			f.Dispose();
			this.Enabled = true;
		}

		private void comboBox_ConnectType_SelectedIndexChanged(object sender, EventArgs e)
		{
			this.Enabled = false;
			listPrinterPort();
			this.Enabled = true;
		}

		private void button_SearchPrinter_Click(object sender, EventArgs e)
		{
			this.Enabled = false;
			listPrinterPort();
			this.Enabled = true;
		}

		private void button_Print_Click(object sender, EventArgs e)
		{
			this.Enabled = false;
			print();
			this.Enabled = true;
		}



		private void print()
		{
			// Get the port type.
			int connectType = getSelectedConnectType();

			// Get the address.
			string address = getSelectedPrinter();

			// Get the layout file.
			string layoutFile = getSelectedLayoutFile();

			// Get Print Option.
			PrintOption option = getPrintOption();
			
			// Get the setting of upside down.
			bool isUpsideDown = checkBox_UpsideDown.Checked;

			// Get the setting of Logo.
			bool isReplaceLogo = checkBox_ReplaceLogo.Checked;

			// Gets the print data.
			String header = textBox_Header.Text;
			String footer = textBox_Footer.Text; ;
			List<string> items = new List<string>();
			items.Add(textBox_Item01.Text);
			items.Add(textBox_Item02.Text);
			items.Add(textBox_Item03.Text);
			items.Add(textBox_Item04.Text);
			items.Add(textBox_Item05.Text);

			doPrint(connectType, address, layoutFile, option, isUpsideDown, isReplaceLogo, header, footer, items.ToArray());
		}


		private void doPrint(int connectType, string address, string layoutFile, com.citizen.sdk.poslayout.PrintOption option, bool isUpsideDown, bool isReplaceLogo, string header, string footer, string[] items)
		{
			int result;
			int frameIndex;
			//-----------------------------------------------------------------
			// Create Layout SDK object.
			// ( for via POS Print SDK )
			//-----------------------------------------------------------------
			com.citizen.sdk.poslayout.LayoutPrintEngine lpe = new com.citizen.sdk.poslayout.LayoutPrintEngine();

			//-----------------------------------------------------------------
			// Open the layout file.
			//-----------------------------------------------------------------
			result = lpe.Open(layoutFile);
			if (LPEConst.CPE_SUCCESS != result)
			{
				string msg = "Please check the layout file. : " + layoutFile;
				Console.WriteLine(msg);
				MessageBoxIcon icon = MessageBoxIcon.Information;
				MessageBox.Show(msg + "  " + "(" + result.ToString() + ")", "Result", MessageBoxButtons.OK, icon);
			}
			else
			{
				//-----------------------------------------------------------------
				// Initialize the object.
				//-----------------------------------------------------------------
				lpe.BeginPrint();

				//-----------------------------------------------------------------
				// Add the header frame(Frame1) to the print area.
				//-----------------------------------------------------------------
				frameIndex = lpe.InitFrame("Frame1");
				result = lpe.SetPartsData(frameIndex, "Text1", header);
				if (isReplaceLogo) {
					// Replace the bitmap.
					string bitmapPath = replaceLogo;
					result = lpe.SetPartsData(frameIndex, "Image1", bitmapPath);
				}
				result = lpe.AddFrame(frameIndex);

				//-----------------------------------------------------------------
				// Replace the text of the item frame(Frame2) and add it to the print area.
				//-----------------------------------------------------------------
				int i = 0;
				foreach (string item in items)
				{
					frameIndex = lpe.InitFrame("Frame2");
					result = lpe.SetPartsData(frameIndex, "Text1", item);
					// Make a dummy price.
					String price = makeDummyPrice(i++);
					result = lpe.SetPartsData(frameIndex, "Text2", price);
					result = lpe.AddFrame(frameIndex);
				}

				//-----------------------------------------------------------------
				// Add the footer frame(Frame3) to the print area.
				//-----------------------------------------------------------------
				frameIndex = lpe.InitFrame("Frame3");
				result = lpe.SetPartsData(frameIndex, "Text1", footer);
				result = lpe.SetPartsData(frameIndex, "Barcode1", footer);
				result = lpe.AddFrame(frameIndex);

				//-----------------------------------------------------------------
				// Initialize the coupon frame(Frame4).
				// Please do not execute AddFrame() method, using InitFrame() method only.
				//-----------------------------------------------------------------
				frameIndex = lpe.InitFrame("Frame4");

				//-----------------------------------------------------------------
				// Print the print area.
				//-----------------------------------------------------------------
				doPrint(connectType, address, lpe, isUpsideDown, option);

				//-----------------------------------------------------------------
				// End processing.
				//-----------------------------------------------------------------
				lpe.EndPrint();
				lpe.Close();
			}
		}
		private int doPrint(int connectType, String address, LayoutPrintEngine lpe, bool isUpsideDown, com.citizen.sdk.poslayout.PrintOption option)
		{
			int result, printResult = -1;

			//
			// Create POS Print SDK object.
			//
			com.citizen.sdk.ESCPOSPrinter printer = new com.citizen.sdk.ESCPOSPrinter();

			// Character set
//			result = printer.SetEncoding("ISO-8859-1");		// Latin-1
			result = printer.SetEncoding("Shift_JIS");		// Japanese

			//
			// Connects to the printer.
			//
			result = printer.Connect(connectType, address);
			if (ESCPOSConst.CMP_SUCCESS == result)
			{
				//
				// Outputs the print data.
				//
				if (option == null)
				{
					printResult = lpe.DoPrint(printer, isUpsideDown);
				}
				else
				{
					printResult = lpe.DoPrint(printer, isUpsideDown, false, option);
				}
				//
				// ESCPOSPrinter error handling
				//
				if (printResult == ESCPOSConst.CMP_E_OFFLINE)
                {
					printer.ClearOutput();
				}
				//
				// Disconnects the printer.
				//
				result = printer.Disconnect();
			}
			else
			{
				// Error
				Console.WriteLine("ESCPOSPrinter connect() error result : " + result);
			}

			return printResult;
		}

		private String makeDummyPrice(int digit)
		{
			// Make a dummy price.  (0.99, 9.99, 99.99,,,)
			double price = Math.Pow(10, digit) - 0.01;
			// Add the comma-separate.
			string s = price.ToString("N");
			return s;
		}


		/// <summary>
		/// Search the printer and to obtain the list of the printer information.
		/// </summary>
		private void listPrinterPort()
		{
			int result;
			int connectType;

			listView_Printer.Columns.Clear();
			listView_Printer.Columns.Add("Address", 200, HorizontalAlignment.Left);
			listView_Printer.Columns.Add("note", -2, HorizontalAlignment.Left);
			//listView_Printer.HeaderStyle = ColumnHeaderStyle.None;
			listView_Printer.Items.Clear();

			try
			{
				Console.WriteLine(comboBox_ConnectType.SelectedValue);
				connectType = (int)comboBox_ConnectType.SelectedValue;
			}
			catch (System.InvalidCastException ex)
			{
				Console.WriteLine(ex + " : " + comboBox_ConnectType.SelectedValue);
				return;
			}

			com.citizen.sdk.ESCPOSPrinter printer = new com.citizen.sdk.ESCPOSPrinter();
			//
			// SearchCitizenPrinter
			//
			CitizenPrinterInfo[] list = printer.SearchCitizenPrinter(connectType, 5, out result);

			if (ESCPOSConst.CMP_SUCCESS != result)
			{
				showWindowsPOSPrintSDKMessage(result);
				return;
			}

			listView_Printer.BeginUpdate();
			for (int i = 0; i < list.Length; i++)
			{
				string[] CLSs = new string[2];
				switch (connectType)
				{
					case ESCPOSConst.CMP_PORT_WiFi:
						CLSs[0] = list[i].ipAddress;
						CLSs[1] = list[i].macAddress;
						break;

					case ESCPOSConst.CMP_PORT_USB:
						CLSs[0] = list[i].deviceName;
						CLSs[1] = list[i].printerModel;
						break;

					case ESCPOSConst.CMP_PORT_COM:
						CLSs[0] = list[i].deviceName;
						break;

					case ESCPOSConst.CMP_PORT_LPT:
						CLSs[0] = list[i].deviceName;
						break;

					case ESCPOSConst.CMP_PORT_Bluetooth:
						CLSs[0] = list[i].deviceName;
						CLSs[0] = list[i].bdAddress;
						break;
				}

				listView_Printer.Items.Add(new ListViewItem(CLSs));
			}
			listView_Printer.EndUpdate();

			listView_Printer.Focus();
			listView_Printer.Items[0].Selected = true;
		}


		void showWindowsPOSPrintSDKMessage(int result)
		{
			string msg = WindowsPOSPrintSDKReturnValue.getDescription(result);

			MessageBoxIcon icon = MessageBoxIcon.Information;
			if (ESCPOSConst.CMP_SUCCESS != result)
			{
				icon = MessageBoxIcon.Error;
			}
			MessageBox.Show(msg + "  " + "(" + result.ToString() + ")", "Result", MessageBoxButtons.OK, icon);
		}
	}


	public class ConnectTypeList : List<KeyValuePair<int, String>>
	{
		public ConnectTypeList()
		{
			Add(new KeyValuePair<int, String>(ESCPOSConst.CMP_PORT_WiFi, "WiFi"));
			Add(new KeyValuePair<int, String>(ESCPOSConst.CMP_PORT_USB, "USB"));
			Add(new KeyValuePair<int, String>(ESCPOSConst.CMP_PORT_COM, "COM"));
			Add(new KeyValuePair<int, String>(ESCPOSConst.CMP_PORT_LPT, "LPT"));
			Add(new KeyValuePair<int, String>(ESCPOSConst.CMP_PORT_Bluetooth, "Bluetooth"));
		}
	}
}
